package be.ASPL.presentationlayer;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Map;

import be.SIRAPRISE.util.MyMessageFormat;

/**
 * TextsEnumerationType is the presentation type for enumerated text values
 * 
 * @author Erwin
 */
public abstract class TextsEnumerationType extends AbstractEnumerationType<StringValue> {

	/**
	 * The textual values of all the enumerated possibilities, with the keys in case-insensitive canonical form
	 */
	private HashMap<String, String> caseInsensitiveValuesAndLabels;

	/**
	 * 
	 */
	private boolean caseSensitive = false;

	/**
	 * The textual values of all the enumerated possibilities
	 */
	private Map<String, String> valuesAndLabels = new HashMap<String, String>();

	/**
	 * Creates the Enumeration type instance
	 * 
	 * @param valuesAndLabels
	 *            The valid values for the enumeration
	 */
	public TextsEnumerationType (Map<String, String> valuesAndLabels) {
		this.valuesAndLabels.putAll(valuesAndLabels);

		HashSet<String> s = new HashSet<String>();
		for (String value : valuesAndLabels.keySet()) {
			String ucValue = value.toUpperCase();
			if (s.contains(ucValue)) {
				caseSensitive = true;
			} else {
				s.add(ucValue);
			}
		}

		if (!caseSensitive) {
			caseInsensitiveValuesAndLabels = new HashMap<String, String>();
			for (String value : valuesAndLabels.keySet()) {
				caseInsensitiveValuesAndLabels.put(value.toUpperCase(), value);
			}
		}
	}

	/**
	 * Gets valuesAndLabels
	 * 
	 * @return valuesAndLabels.
	 */
	final Map<String, String> getValuesAndLabels ( ) {
		return valuesAndLabels;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#checkValidWaarValue(be.ASPL.presentationlayer.WaarValue, be.ASPL.presentationlayer.UserData)
	 */
	@Override
	public final StringValue checkValidWaarValue (PresentationValue value, UserData userData) throws InvalidAttributeValueException {
		if (!(value instanceof StringValue)) {
			Locale locale = userData.getLocale();
			throw new InvalidAttributeValueException(value.toASPLString(userData), this.getClass().getSimpleName(), MyMessageFormat.format(I18N.getString(locale, "DynamicColumnList.InvalidValueClass"), new String[] { getClass().getName(), value.getClass().getName() })); //$NON-NLS-1$
		}

		return encode(((StringValue) value).getString(), userData, userData.getLocale());

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#encode(byte[], be.ASPL.presentationlayer.UserData, java.util.Locale)
	 */
	@Override
	public StringValue encode (byte[] partBytes, UserData userData, Locale locale) {
		throw new UnsupportedOperationException();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.waar.AttributeType#validate(java.lang.String)
	 */
	public final StringValue encode (String val, UserData userData, Locale locale) throws InvalidAttributeValueException {
		if (caseSensitive) {
			String encodeKey = val;
			if (valuesAndLabels.containsKey(encodeKey)) {
				return new StringValue(val);
			} else {
				throw new InvalidAttributeValueException(val, this.getClass().getSimpleName(), valuesAndLabels.keySet().toString());
			}
		} else {
			String encodeKey = val.toUpperCase();
			if (caseInsensitiveValuesAndLabels.containsKey(encodeKey)) {
				return new StringValue(caseInsensitiveValuesAndLabels.get(encodeKey));
			} else {
				throw new InvalidAttributeValueException(val, this.getClass().getSimpleName(), valuesAndLabels.keySet().toString());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getDefaultPresentationRenderingStyle()
	 */
	@Override
	public final Renderer getDefaultPresentationRenderingStyle ( ) {
		return ComboBoxRenderer.getInstance();
	}

	public String getEmptyNakedStringRepresentation (PresentationField presentationField, Locale locale) {
		return ""; //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.waar.AttributeType#validate(java.lang.String)
	 */
	public final String getNakedStringRepresentation (PresentationValue presentationValue, OnLinePresentationFunction userFunction, PresentationField presentationField, Locale locale) {
		String stringValue = ((StringValue) presentationValue).getString();
		if (caseSensitive) {
			String label = valuesAndLabels.get(stringValue);
			return (label == null ? "" : label); //$NON-NLS-1$
		} else {
			String label = caseInsensitiveValuesAndLabels.get(stringValue.toUpperCase());
			return (label == null ? "" : label); //$NON-NLS-1$
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.erwinsmout.ASPL.presentationlayer.PresentationType#getPresentationRenderingStyle(java.lang.String)
	 */
	@Override
	public final Renderer getPresentationRenderingStyle (String renderingStyleName) {
		if (renderingStyleName.equalsIgnoreCase("PASSWORD")) { //$NON-NLS-1$
			return PassWordRenderer.getInstance();
		}
		if (renderingStyleName.equalsIgnoreCase("TEXTBOX")) { //$NON-NLS-1$
			return TextBoxRenderer.getInstance();
		}
		return ComboBoxRenderer.getInstance();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see be.ASPL.presentationlayer.PresentationType#getWaarValue(boolean, java.util.Locale)
	 */
	@Override
	public final PresentationValue getWaarValue (boolean b, Locale locale) {
		throw new UnsupportedOperationException();
	}

	@Override
	public final PresentationValue getWaarValue (int v, Locale locale) {
		throw new UnsupportedOperationException();
	}

	@Override
	public final PresentationValue getWaarValue (long v, Locale locale) {
		throw new UnsupportedOperationException();
	}

}
